@@ -1,20 +1,23 @@
 # -*- coding: utf-8 -*-
 
 """
-Usage: process_mails [-h] FILE [--requeue_errors] [--list_emails] [--get_eml=<mail_id>]  [--gen_pdf=<mail_id>] [--get_eml_orig]
+Usage: process_mails FILE [--requeue_errors] [--list_emails=<number>] [--get_eml=<mail_id>]  [--gen_pdf=<mail_id>]
+                          [--get_eml_orig]
 
 Arguments:
     FILE         config file
 
 Options:
-    -h --help           Show this screen.
-    --requeue_errors    Put email in error status back in waiting for processing
-    --list_emails       List last 20 emails
-    --get_eml=<mail_id> Get eml of original/contained email id
-    --get_eml_orig      Get eml of original email id (otherwise contained)
-    --gen_pdf=<mail_id> Generate pdf of contained email id
+    -h --help               Show this screen.
+    --requeue_errors        Put email in error status back in waiting for processing.
+    --list_emails=<number>  List last xx emails.
+    --get_eml=<mail_id>     Get eml of original/contained email id.
+    --get_eml_orig          Get eml of original email id (otherwise contained).
+    --gen_pdf=<mail_id>     Generate pdf of contained email id.
 """
+
 from datetime import datetime
+from datetime import timedelta
 from docopt import docopt
 from email.mime.base import MIMEBase
 from email.mime.multipart import MIMEMultipart
@@ -27,6 +30,7 @@ from imio.email.parser.parser import Parser  # noqa
 from io import BytesIO
 from smtplib import SMTP
 import configparser
+import imaplib
 import json
 import logging
 import os
@@ -104,7 +108,7 @@ def notify_exception(config, mail_id, mail, error):
     elif hasattr(error, 'reason'):
         try:
             error_msg = u"'{}', {}, {}, {}".format(error.reason, error.start, error.end, error.object)
-        except:
+        except Exception:
             error_msg = error.reason
     main_text = MIMEText(ERROR_MAIL.format(client_id, login, mail_id, error.__class__, error_msg), "plain")
     msg.attach(main_text)
@@ -260,7 +264,7 @@ def process_mails():
         lock.close()
         sys.exit()
     elif arguments.get("--list_emails"):
-        handler.list_last_emails()
+        handler.list_last_emails(nb=int(arguments.get("--list_emails")))
         # import ipdb; ipdb.set_trace()
         # handler.mark_reset_error('58')
         handler.disconnect()
@@ -325,9 +329,72 @@ def process_mails():
             errors += 1
 
     logger.info(
-        "{} emails have been imported. {} email are unsupported. {} emails have caused an error.".format(
+        "{} emails have been imported. {} emails are unsupported. {} emails have caused an error.".format(
             imported, unsupported, errors
         )
     )
     handler.disconnect()
     lock.close()
+
+
+def clean_mails():
+    """Clean mails from imap box.
+
+    Usage: clean_mails FILE [-h] [--kept_days=<number>] [--ignored_too] [--list_only]
+
+    Arguments:
+        FILE         config file
+
+    Options:
+        -h --help               Show this screen.
+        --kept_days=<number>    Days to keep [default: 30]
+        --ignored_too           Get also not imported emails
+        --list_only             Only list related emails, do not delete
+    """
+    arguments = docopt(clean_mails.__doc__)
+    config = configparser.ConfigParser()
+    config.read(arguments["FILE"])
+    days = int(arguments["--kept_days"])
+    doit = not arguments["--list_only"]
+    host, port, ssl, login, password = get_mailbox_infos(config)
+    handler = IMAPEmailHandler()
+    handler.connect(host, port, ssl, login, password)
+    before_date = (datetime.now() - timedelta(days)).strftime("%d-%b-%Y")  # date string 01-Jan-2021
+    # before_date = '01-Jun-2021'
+    res, data = handler.connection.search(None, '(BEFORE {0})'.format(before_date))
+    if res != "OK":
+        logger.error("Unable to fetch mails before '{}'".format(before_date))
+        handler.disconnect()
+        sys.exit()
+    deleted = ignored = error = 0
+    mail_ids = data[0].split()
+    logger.info("Get '{}' emails older than '{}'".format(len(mail_ids), before_date))
+    # sys.exit()
+    for mail_id in mail_ids:
+        res, flags_data = handler.connection.fetch(mail_id, '(FLAGS)')
+        if res != "OK":
+            logger.error("Unable to fetch flags for mail {0}".format(mail_id))
+            error += 1
+            continue
+        flags = imaplib.ParseFlags(flags_data[0])
+        if not arguments["--ignored_too"] and b"imported" not in flags:
+            ignored += 1
+            continue
+        mail = handler.get_mail(mail_id)
+        if not mail:
+            error += 1
+            continue
+        parser = Parser(mail)
+        logger.info(u"{}: '{}'".format(mail_id, parser.headers['Subject']))
+        if doit:
+            handler.connection.store(mail_id, "+FLAGS", "\\Deleted")
+        deleted += 1
+    if deleted:
+        logger.info("Get '{}' emails older than '{}'".format(len(mail_ids), before_date))
+        if doit:
+            res, data = handler.connection.expunge()
+            if res != "OK":
+                logger.error("Unable to deleted mails")
+    handler.disconnect()
+    logger.info("{} emails have been deleted. {} emails are ignored. {} emails have caused an error.".format(
+                deleted, ignored, error))
